package tools

import (
	"errors"
	"sync"

	"xbus.io/go-xbus/v3/api"
	"xbus.io/go-xbus/v3/api/control"
)

// ErrNoSuchPipeline is returned if the pipeline does not exist
var ErrNoSuchPipeline = errors.New("no such pipeline")

// NewPipelineCache creates a PipelineCache instance
func NewPipelineCache(control *control.Client) *PipelineCache {
	return &PipelineCache{
		control: control,
		cache:   make(map[api.UUID]api.PipelineInfo),
	}
}

// PipelineCache caches the pipeline definitions
type PipelineCache struct {
	control *control.Client
	cache   map[api.UUID]api.PipelineInfo
	m       sync.Mutex
}

// Get returns a pipeline info
func (pc *PipelineCache) Get(id api.UUID) (api.PipelineInfo, error) {
	pc.m.Lock()
	defer pc.m.Unlock()
	if p, ok := pc.cache[id]; ok {
		return p, nil
	}
	rep, err := pc.control.Pipeline.Query(control.PipelineQueryRequest{})
	if err != nil {
		return api.PipelineInfo{}, err
	}
	for _, p := range rep.GetPipelineList() {
		pc.cache[p.ID] = p
	}
	if p, ok := pc.cache[id]; ok {
		return p, nil
	}
	return api.PipelineInfo{}, ErrNoSuchPipeline
}
