package metrics

import (
	"time"

	"xbus.io/go-xbus/v3/api"
	"xbus.io/go-xbus/v3/api/control"

	"xbus.io/xbus/storage"
)

func NewEnvelopeStorage(name, implementation string, backend storage.EnvelopeStorage) *EnvelopeStorage {
	return &EnvelopeStorage{
		timer:   NewStorageTimer("envelope", name, implementation),
		backend: backend,
	}
}

type EnvelopeStorage struct {
	timer   StorageTimer
	backend storage.EnvelopeStorage
}

func (s EnvelopeStorage) SetEventHandler(handler storage.EnvelopeStorageEventHandler) {
	defer s.timer.Time("SetEventHandler")(true)
	s.backend.SetEventHandler(handler)
}

func (s EnvelopeStorage) StoreEnvelope(id api.Envelope) (storage.EnvelopeState, error) {
	observe := s.timer.Time("StoreEnvelope")
	state, err := s.backend.StoreEnvelope(id)
	observe(err == nil)
	return state, err
}

func (s EnvelopeStorage) CheckStalled(id api.UUID, duration time.Duration) bool {
	defer s.timer.Time("CheckStalled")(true)
	return s.backend.CheckStalled(id, duration)
}

func (s EnvelopeStorage) GetEventTypes(id api.UUID) ([]string, bool) {
	defer s.timer.Time("GetEventTypes")(true)
	return s.backend.GetEventTypes(id)
}

// ReceptionStatus returns the envelope reception status
func (s EnvelopeStorage) ReceptionStatus(id api.UUID) (
	storage.EnvelopeStorageReceptionStatus, error,
) {
	observe := s.timer.Time("ReceptionStatus")
	status, err := s.backend.ReceptionStatus(id)
	observe(err == nil)
	return status, err
}

func (s EnvelopeStorage) ReadEnvelope(
	id api.UUID, position storage.EnvelopeReadPos, maxsize int,
) (api.Envelope, storage.EnvelopeReadPos, error) {
	observe := s.timer.Time("ReadEnvelope")
	e, p, err := s.backend.ReadEnvelope(id, position, maxsize)
	observe(err == nil)
	return e, p, err
}

func (s EnvelopeStorage) Purge(filter storage.EnvelopeFilter) {
	defer s.timer.Time("Purge")(true)
	s.backend.Purge(filter)
}

func (s EnvelopeStorage) Stat() ([]control.StorageStat_Entry, error) {
	observe := s.timer.Time("Stat")
	stat, err := s.backend.Stat()
	observe(err == nil)
	return stat, err
}
