package withfallback

import (
	"xbus.io/go-xbus/v3/api"
	"xbus.io/go-xbus/v3/api/control"

	"xbus.io/xbus/storage"
)

func NewEnvelopeStorage(
	primary, fallback storage.EnvelopeStorage,
) EnvelopeStorage {
	return EnvelopeStorage{
		primary, fallback,
	}
}

type EnvelopeStorage struct {
	storage.EnvelopeStorage
	fallback storage.EnvelopeStorage
}

func (s EnvelopeStorage) GetEventTypes(id api.UUID) ([]string, bool) {
	types, exists := s.EnvelopeStorage.GetEventTypes(id)
	if !exists {
		types, exists = s.fallback.GetEventTypes(id)
	}
	return types, exists
}

func (s EnvelopeStorage) ReceptionStatus(id api.UUID) (storage.EnvelopeStorageReceptionStatus, error) {
	status, err := s.EnvelopeStorage.ReceptionStatus(id)
	// TODO we need a ErrNoEnvelope error to avoid skipping important errors
	if err != nil {
		status, err = s.fallback.ReceptionStatus(id)
	}
	return status, err
}

func (s EnvelopeStorage) ReadEnvelope(
	id api.UUID, position storage.EnvelopeReadPos, maxsize int,
) (api.Envelope, storage.EnvelopeReadPos, error) {
	e, p, err := s.EnvelopeStorage.ReadEnvelope(id, position, maxsize)
	if err != nil {
		e, p, err = s.fallback.ReadEnvelope(id, position, maxsize)
	}
	return e, p, err
}

func (s EnvelopeStorage) Purge(filter storage.EnvelopeFilter) {
	s.EnvelopeStorage.Purge(filter)
	s.fallback.Purge(filter)
}

func (s EnvelopeStorage) Stat() ([]control.StorageStat_Entry, error) {
	primary, err := s.EnvelopeStorage.Stat()
	if err != nil {
		return nil, err
	}
	fallback, err := s.fallback.Stat()
	if err != nil {
		return nil, err
	}

	var stats = make([]control.StorageStat_Entry, len(primary))
	copy(stats, primary)
	stats = append(stats, fallback...)

	var total = control.StorageStat_Entry{
		Implementation: "withfallback",
	}
	total.Name = "("
	for i, s := range primary {
		if i != 0 {
			total.Name += ", "
		}
		total.Name += s.Name
	}
	total.Name += ") withfallback ("
	for i, s := range fallback {
		if i != 0 {
			total.Name += ", "
		}
		total.Name += s.Name
	}
	total.Name += ")"

	stats = append(stats, total)
	return stats, nil
}
